package com.contextspace;

import static org.junit.Assert.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.ElementCollection;
import javax.persistence.Embeddable;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.Persistence;
import javax.persistence.Table;
import javax.persistence.TableGenerator;
import javax.persistence.TypedQuery;

import org.junit.Test;

public class TestEmbeddable {

    @Test
    public void test() {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("objectdb:$objectdb/db/test.tmp;drop");
        EntityManager em = emf.createEntityManager();
     
        
        em.getTransaction().begin();
        
        Rule newRule = new Rule("Test rule");
        
        RulePart part1 = new RulePart();
        part1.setAttributeName("attr1");
        part1.setOperation("equals");
        part1.setValue("value");
        
        RulePart part2 = new RulePart();
        part2.setAttributeName("attr2");
        part2.setOperation("<");
        part2.setValue("1");
        
        newRule.addRulePart(part1);
        newRule.addRulePart(part2);
        
        newRule.setCompiledRule("Some compiled rule");
        
        em.persist(newRule);
        em.getTransaction().commit();
        em.flush();
        
        TypedQuery<Long> query = em.createQuery("SELECT COUNT($1) FROM Rule $1", Long.class);
        Long ruleCount = query.getSingleResult();
        assertEquals(1, ruleCount.intValue());
        
        TypedQuery<Rule> query2 = em.createQuery("SELECT $1 FROM Rule $1", Rule.class);
        List<Rule> ruleList = query2.getResultList();
        assertEquals(1, ruleList.size());
        
        Rule retrievedRule = ruleList.get(0);
        assertNotNull(retrievedRule.getRules());
        assertEquals(2, retrievedRule.getRules().size());
        assertEquals("Test rule", retrievedRule.getName());
        
        
        em.close();
        emf.close();
    }
    
    
    
    @Table
    @Entity
    @Inheritance(strategy=InheritanceType.TABLE_PER_CLASS)
    @TableGenerator(name="IdTableGen", initialValue=1, allocationSize=50)
    public static class Rule implements Serializable {

        private static final long serialVersionUID = 1L;
        private String name = null;
        
        List<RulePart> rules = new ArrayList<RulePart>();
        
        private String compiledRule = null;

        /**
         * Default Constructor
         */
        public Rule() {
            
        }
        
        /**
         * Constructor
         * @param name
         * @param category
         */
        public Rule(String name) {
            this.name = name;
        }
        
        private long id;

        /**
         * @return the object id
         */
        @Id
        @GeneratedValue(strategy=GenerationType.TABLE, generator="IdTableGen")
        public long getId() {
            return id;
        }

        /**
         * @param id the id to be set
         */
        public void setId(long id) {
            this.id = id;
        }
        
        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param name the name to set
         */
        public void setName(String name) {
            this.name = name;
        }

        /**
         * @return the rules
         */
        //@ElementCollection
        @Embedded
        public List<RulePart> getRules() {
            return rules;
        }

        /**
         * @param rules the rules to set
         */
        public void setRules(List<RulePart> rules) {
            this.rules = rules;
        }
        
        /**
         * Add rulePart
         * @param newPart
         */
        public void addRulePart(RulePart newPart) {
            this.rules.add(newPart);
        }

        /**
         * @return the compiledRule
         */
        public String getCompiledRule() {
            return compiledRule;
        }

        /**
         * @param compiledRule the compiledRule to set
         */
        public void setCompiledRule(String compiledRule) {
            this.compiledRule = compiledRule;
        }
    }
    
    
    @Embeddable
    public static class RulePart {

        private String attributeName = null;
        private String operation = null;
        private String value = null;
        
        /**
         * Default Constructor
         */
        public RulePart() {
            
        }
        
        /**
         * @return the attributeName
         */
        public String getAttributeName() {
            return attributeName;
        }
        /**
         * @param attributeName the attributeName to set
         */
        public void setAttributeName(String attributeName) {
            this.attributeName = attributeName;
        }
        /**
         * @return the operation
         */
        public String getOperation() {
            return operation;
        }
        /**
         * @param operation the operation to set
         */
        public void setOperation(String operation) {
            this.operation = operation;
        }
        /**
         * @return the value
         */
        public String getValue() {
            return value;
        }
        /**
         * @param value the value to set
         */
        public void setValue(String value) {
            this.value = value;
        }
    }
    
}
